#############################
# Kidney Consomics
# Churchill/Shockley
# Modified September 14, 2005
# R/maanova version 0.98-7 
#############################

# Set the working directory. Include the .CEL files in working directory.
setwd("C:/TEST_CSSpaper");

# load R/affyPLM
library(affyPLM);

RmaProcessedData=justRMA(); 
ExpressionOfRmaProcData=exprs(RmaProcessedData);
ProcessedResult=data.frame(cloneid=row.names(ExpressionOfRmaProcData), ExpressionOfRmaProcData);
write.table(ProcessedResult,"data.norm_kidney_samples.dat", sep="\t", row=F, quote=F);
design.table=data.frame(Array = row.names(pData(RmaProcessedData)));
write.table(design.table,"design_kidney_samples.dat", sep="\t", row=F, quote=F);

# Make sure to modify the design file appropriately. Check the R/maanova help file for instructions, if necessary.

################################
# ANOVA Analysis - permutation
# F-tests
################################

# load R/maanova
library(maanova);

# Place the data files in the working directory
raw.data = read.madata("data.norm_kidney_samples.dat", designfile="design_kidney_samples.dat", cloneid=1, pmt=2, spot=F);
data = createData(raw.data, n.rep=1, log.tran=F);

# Subset the data into strain groupings
data.B6B1 = subset.madata(data, c(5:8, 1:4));
data.B6B6 = subset.madata(data, c(9:12, 1:4));
data.B6B15 = subset.madata(data, c(13:16, 1:4));
DATA = list(data.B6B1, data.B6B6, data.B6B15);

ftest.strain.Pvalperm = matrix(0, nrow=45101, ncol=3); colnames(ftest.strain.Pvalperm) = c("1", "6", "15");
for(i in 1:3){  #just do 3 tests in the kidney
         model.tmp = makeModel(data=DATA[[i]], formula=~Sex + Strain);
         test.tmp = matest(DATA[[i]], model.tmp, term="Strain", shuffle.method="resid", n.perm=1000, pval.pool=TRUE);
         ftest.strain.Pvalperm[,i] = test.tmp$Fs$Pvalperm;
}

ftest.sex.Pvalperm = matrix(0, nrow=45101, ncol=3); colnames(ftest.sex.Pvalperm) = c("1", "6", "15");
for(i in 1:3){
         model.tmp = makeModel(data=DATA[[i]], formula=~Sex + Strain);
         test.tmp = matest(DATA[[i]], model.tmp, term="Sex", shuffle.method="resid", n.perm=1000, pval.pool=TRUE);
         ftest.sex.Pvalperm[,i] = test.tmp$Fs$Pvalperm;
}

ftest.int.Pvalperm = matrix(0, nrow=45101, ncol=3); colnames(ftest.int.Pvalperm) = c("1", "6", "15");
for(i in 1:3){
         model.tmp = makeModel(data=DATA[[i]], formula=~Sex + Strain + Sex:Strain);
         test.tmp = matest(DATA[[i]], model.tmp, term="Sex:Strain", shuffle.method="resid", n.perm=1000, pval.pool=TRUE);
         ftest.int.Pvalperm[,i] = test.tmp$Fs$Pvalperm;
}

#########################
# Perform FDR calculation
#########################

# load R/qvalue
library(qvalue);

vector.strain = as.vector(ftest.strain.Pvalperm);
q.pool.strain.tmp = qvalue(vector.strain)$qvalues
q.pool.strain = matrix(q.pool.strain.tmp, ncol=3);

vector.sex = as.vector(ftest.sex.Pvalperm);
q.pool.sex.tmp = qvalue(vector.sex)$qvalues;
q.pool.sex = matrix(q.pool.sex.tmp, ncol=3);

vector.int = as.vector(ftest.int.Pvalperm);
q.pool.int.tmp = qvalue(vector.int)$qvalues;
q.pool.int = matrix(q.pool.int.tmp, ncol=3);

# Load Affymetrix chromosome bp positions. This file was prepared by parsing the Affymetrix annotation file. 
chrmapping = read.csv("affy.CHR.bp.positions.090605.csv");
chrmapping = data.frame(chrmapping, q.pool.sex, q.pool.strain, q.pool.int);

#######################################################################
# Find overlap between sum Probe IDs for each test with each other test
#######################################################################
qcut = 0.01
allchr = chrmapping[,2] != "Un" & chrmapping[,2] != "0" & chrmapping[,2] != "M";
pos=8; # position where strain test begins
sumintersections = matrix(NA, nrow=3, ncol=3);
for(j in 1:3){
    for(i in 1:3){
        sumintersections[i,j] = length(intersect(data$cloneid[chrmapping[,pos+j-1] < qcut & allchr],
                                               data$cloneid[chrmapping[,pos+i-1] < qcut & allchr]))
    }
}
colnames(sumintersections) = c("B6.BA1", "B6.BA6", "B6.BA15");
rownames(sumintersections) = c("B6.BA1", "B6.BA6", "B6.BA15");

###################################
# Chi-square test for intersections
###################################
Chi.test = matrix(NA, nrow=3, ncol=3);
for(i in 1:3){ 
    for(j in 1:3){
        C.temp = matrix(c(sumintersections[i,j], sumintersections[i,i] - sumintersections[i,j],
        sumintersections[j,j] - sumintersections[i,j], 45101-sumintersections[i,i]-sumintersections[j,j]), nrow=2)
        Chi.test[i,j] = chisq.test(C.temp)$p.value
    }
}

Chi.test[upper.tri(Chi.test, diag=FALSE)] <- NA
Chi.test=round(Chi.stat, 4);

colnames(Chi.test) = c("B.A1", "B.A6", "B.A15");
rownames(Chi.test) = c("B.A1", "B.A6", "B.A15");

#################################
# Fisher's test for intersections
#################################
Fisher.test = matrix(NA, nrow=3, ncol=3);
for(i in 1:3){
    for(j in 1:3){
        C.temp = matrix(c(sumintersections[i,j], sumintersections[i,i] - sumintersections[i,j],
        sumintersections[j,j] - sumintersections[i,j],
        45101 - (sumintersections[i,i]-sumintersections[i,j]) - (sumintersections[j,j] - sumintersections[i,j])),
        nrow=2)
    Fisher.test[i,j] = fisher.test(C.temp)$p.value
    }
}

#Fisher.test[upper.tri(Fisher.test, diag=FALSE)] <- NA
#Fisher.test=round(Fisher.test, 4);

colnames(Fisher.test) = c("B.A1", "B.A6", "B.A15");
rownames(Fisher.test) = c("B.A1", "B.A6", "B.A15");

#################
# Pval histograms
#################
par(mfcol=c(2,2));
for(i in 1:3){
    hist(ftest.strain.Pvalperm[,i], breaks=500, main = "",
         ylim=c(0,700), xlab=c("Pvalperm"), font.lab=2, font.axis=2);
    mtext("kidney - test strain", line = -7.5, at=0.35, cex=0.75, font=2, adj=0, col="blue");
    mtext(paste(c("DE Probe IDs [q<0.01]- "), sum(q.pool.strain[,i] < 0.01)), line = -6, at=0.1,
         cex=0.65, font=2, adj=0, col="red");
    mtext(c("B6.BA1", "B6.BA6", "B6.BA15")[i], line = 0, at=0.35, cex=1, font=2, adj=0, col="black");
}    

######
# pi-0
######
pi_1= matrix(NA, nrow=3, ncol=5);
for(i in 1:3){
    pi_1[i,1] = 1 - qvalue(ftest.strain.Pvalperm[,i], lambda=0.5)$pi0
    pi_1[i,2] = 1 - qvalue(ftest.strain.Pvalperm[,i], lambda=0.6)$pi0
    pi_1[i,3] = 1 - qvalue(ftest.strain.Pvalperm[,i], lambda=0.7)$pi0
    pi_1[i,4] = 1 - qvalue(ftest.strain.Pvalperm[,i], lambda=0.8)$pi0
    pi_1[i,5] = 1 - qvalue(ftest.strain.Pvalperm[,i], lambda=0.9)$pi0
}    
colnames(pi_1) = c("Pvalperm_lambda.5", "Pvalperm_lambda.6", "Pvalperm_lambda.7", "Pvalperm_lambda.8",
                   "Pvalperm_lambda.9");
rownames(pi_1) = c("B6.BA1", "B6.BA6", "B6.BA15");

#################################
# Plot distribution by chromosome
#################################
chromosomes = c("1", "2", "3", "4", "5", "6", "7", "8", "9", "10", "11", "12", "13", "14", "15", "16",
                "17", "18", "19", "X", "Y");
par(mfrow=c(3,3))
test = 1
for(i in 1:21){
    hist(ftest.strain.Pvalperm[,test][chrmapping[,2]==chromosomes[i]], xlab = "Pvalperm",
         main = paste("Test Strain, chr ", chromosomes[i]), breaks=200, ylim=c(0,250));
    mtext(paste("total on CHR = ", sum(chrmapping[,2] == chromosomes[i])),
          line=-3.5, at=0.15, cex=0.5, font=1, adj=0);
    mtext(paste("sig. on CHR = ", sum(ftest.strain.Pvalperm[,test][chrmapping[,2]==chromosomes[i]] < 0.001)),
          line=-4.5, at=0.15, cex=0.5, font=1, adj=0);
    if(i/9 == as.integer(i/9)){x11(); par(mfcol=c(3,3))}
}

#########################################################
# Calculate Fold Changes in Expression Between B and B.AN
#########################################################
FC.strain = matrix(NA, nrow=45101, ncol=3);
     
for(i in 1:3){
    model.temp1 = makeModel(data=DATA[[i]], formula=~Sex + Strain);
    temp = fitmaanova(DATA[[i]], model.temp1);
         for(j in 1:45101){
             if(temp$Strain[j,1] > temp$Strain[j,2] & i != 20 & i !=21){
                FC.strain[j,i] = 2^abs(temp$Strain[j,1] - temp$Strain[j,2])
             }
             if(temp$Strain[j,1] < temp$Strain[j,2] & i != 20 & i != 21){
                FC.strain[j,i] = -1*2^abs(temp$Strain[j,1] - temp$Strain[j,2])
             }
             if(i == 20 | i == 21){
                if(temp$Strain[j,1] > temp$Strain[j,2]){
                   FC.strain[j,i] = -1*2^abs(temp$Strain[j,1] - temp$Strain[j,2])
                }
                if(temp$Strain[j,1] < temp$Strain[j,2]){
                   FC.strain[j,i] = 2^abs(temp$Strain[j,1] - temp$Strain[j,2])
                }
             }
         }
    print(i)
}

# Calculate the number of DE probe sets across all B.AN strains
tally.3 = vector(length=0);
for(i in 1:45101){tally.3[i] = sum(q.pool.strain[i,1:3] < 0.01)}

#Which chr do changes occur on?
tally.3.sum = matrix(NA, nrow=3, ncol=22);
for(j in 1:3){        #strains
    for(i in 1:21){    #chromosomes
        tally.3.chr = chrmapping[,2][which(tally.3 > j-1)];
        tally.3.sum[j,i] = sum(tally.3.chr == chromosomes[i])
    }
}
rownames(tally.3.sum) = 1:3
colnames(tally.3.sum) = c(chromosomes, "sum");
for(i in 1:3){tally.3.sum[i, 22] = sum(tally.3.sum[i,1:3])}

# Find probe sets for unique chr that appear DE in at least 1 list
listlocations = which(tally.3 >= 1 & chrmapping[, 2] != 0 & chrmapping[,2] != "Un") #use tally.3 for locations
probeid.lists = data$cloneid[listlocations];
tally.3.lists = tally.3[listlocations];
chr.lists = chrmapping[listlocations, 2];
q.pool.strain.lists = matrix(q.pool.strain[listlocations, 1:3])
FC.lists = matrix(FC.strain[listlocations, 1:3]);
tally.lists = data.frame(probeid.lists, chr.lists, tally.3.lists);

# Direction of Change for probe sets with unique chr that appear DE in at least 1 list
CSS_focus = list(length=0);
withCSS = list(length=0); 
for(j in 1:3){
    withCSS[[3]] = vector(length=0);
    CSS_focus = j
    DEgene = list(length=0);
    q.list = list(length=0);
    FC = list(length=0);
    x=1;
    for(i in 1:length(tally.3.lists)){
        DEgene[[i]] = which(q.pool.strain.lists[i,] < 0.01) #if a gene is DE, which strain tests are DE?
        FC[[i]] = FC.lists[i, DEgene[[i]]];
        q.list[[i]] = q.pool.strain.lists[i,DEgene[[i]]];
        if(any(DEgene[[i]] == CSS_focus)){
           withCSS[[j]][x] = i;
           x = x + 1
        }
    }
}

all = list(length=0);
cis = list(length=0);
trans=list(length=0);
chrtest = c("1", "6", "15");

#################################################################     
# Distribution of cis and trans-eQTL for different FDR thresholds
# Red = cis-QTL, Blue = trans-QTL
#################################################################
qcut = c(0.001, 0.005, 0.01, 0.05, 0.10, 0.20, 0.5, 0.75, 1.0);

for(k in 1:length(qcut)){
    strain.norm = matrix(NA, nrow=3, ncol=23);
    colnames(strain.norm) = c("1", "2", "3", "4", "5", "6", "7", "8", "9", "10", "11", "12", "13", "14", "15", "16", "17",
                          "18", "19", "X", "Y", "SUM", "% in CHR"); #Columns are chromosomes
    rownames(strain.norm) = c("B.A1", "B.A6", "B.A15");  #rows are tests (test 1, test 2, etc.)

    for(i in 8:10){
        for(j in 1:21){
            strain.norm[i-7, j] = sum(chrmapping[,2] == chromosomes[j] & chrmapping[,i] < qcut[k])/chrsize[j]
    }
    strain.norm[i-7, 22] = sum(strain.norm[i-7,1:21]);
    if((i-7)< 22){strain.norm[i-7, 23] =
        100/sum(strain.norm[i-7, 1:21])*strain.norm[i-7, which(colnames(strain.norm) == chromosomes[i-7])]};
    }

    all[[k]] = strain.norm[c(1:3), c(1:21)]
    cis[[k]] = vector(length=0); 
    for(i in 1:3){cis[[k]][i] = strain.norm[i, chrtest[i]]}

    trans[[k]] = vector(length=0);
    x=1;
    for(i in 1:3){
        for(j in 1:21){
            if(i != j){trans[[k]][x] = strain.norm[i,j]; x=x+1}
        }
    }

    dest <- density(all[[k]])  # or some smaller width, maybe,
    hist(all[[k]], xlim = range(dest$x), xlab = "x", ylab = "density",
        probability = TRUE, lwd=2, main=paste("q <", qcut[k]), col="white", border="white")
    lines(dest, lty=2)
    lines(density(cis[[k]]), lwd=2, col=2);
    lines(density(trans[[k]]), lwd=2, col=4)
}

#################################
# PLOT ALL q-values on same graph
#################################
IQR.all = list(length=0);
IQR.trans = list(length=0);
IQR.cis = list(length=0);

for(k in 1:9){
IQR.all[[k]] <- abs(diff(summary(as.vector(all[[k]]))[c(5,2)]));
IQR.trans[[k]] <- abs(diff(summary(as.vector(trans[[k]]))[c(5,2)]));
IQR.cis[[k]] <- abs(diff(summary(as.vector(cis[[k]]))[c(5,2)]));
}

tuning = c(800, 450, 400, 200, 150, 90, 35, 25, 1.5);

par(mfrow=c(3,3));
for(j in 1:length(qcut)){
    dest <- density(all[[j]])  # or some smaller width, maybe
    hist(all[[j]], xlim = range(dest$x), xlab = "(No. DE probe sets)/(chr length)", ylab = "density", ylim=c(0,tuning[j]),
           probability = TRUE, lwd=2, main=paste("q <", qcut[j]), col="white", border="white");
    if(IQR.all[[j]] > 0){
       lines(density(cis[[j]], width=3*IQR.cis[[j]]), lwd=2, col=2);
       lines(dest, lty=2, lwd=2)
       lines(density(trans[[j]], width=3*IQR.trans[[j]]), lwd=2, col=4);
    }
    if(IQR.all[[j]] == 0){
       lines(density(cis[[j]]), lwd=2, col=2);
       lines(density(all[[j]]), lty=2, lwd=2)
       lines(density(trans[[j]]), lwd=2, col=4);
    }
}